# coding=utf-8

import unittest
import os
from ddt import ddt, data, unpack
from elifetools import utils_html


@ddt
class TestUtilsHtml(unittest.TestCase):
    def setUp(self):
        self.mock_tag = type("", (object,), {})
        setattr(self.mock_tag, "name", "foo")

    @unpack
    @data(
        (False, None, None, None),
        (True, None, None, None),
        (
            True,
            u"<p><bold>A</bold> <italic>α</italic> <underline>c</underline></p>",
            None,
            u'<p><b>A</b> <i>α</i> <span class="underline">c</span></p>',
        ),
        (
            False,
            u"<p><bold>A</bold> <italic>α</italic> <underline>c</underline></p>",
            None,
            u"<p><bold>A</bold> <italic>α</italic> <underline>c</underline></p>",
        ),
        (
            True,
            u'<p><bold>A</bold> 1 &lt; 2 &gt; 1 <xref rid="bib1">&gt;α&lt;</xref>&gt;</p>',
            None,
            u'<p><b>A</b> 1 &lt; 2 &gt; 1 <a href="#bib1">&gt;α&lt;</a>&gt;</p>',
        ),
        (
            True,
            u"A bad xref <xref>&gt;α&lt;</xref>",
            None,
            u"A bad xref <xref>&gt;α&lt;</xref>",
        ),
        (
            True,
            u'Link 1 <ext-link ext-link-type="uri" xlink:href="http://example.org/example.html">http://example.org/example.html</ext-link>',
            None,
            u'Link 1 <a href="http://example.org/example.html">http://example.org/example.html</a>',
        ),
        (
            True,
            u'Link 2 <ext-link ext-link-type="doi" xlink:href="10.7554/eLife.00001.012">http://dx.doi.org/10.7554/eLife.00001.012</ext-link>',
            None,
            u'Link 2 <a href="https://doi.org/10.7554/eLife.00001.012">http://dx.doi.org/10.7554/eLife.00001.012</a>',
        ),
        (
            True,
            u'Bad link 1 <ext-link xlink:href="10.7554/eLife.00001.012">http://dx.doi.org/10.7554/eLife.00001.012</ext-link>',
            None,
            u'Bad link 1 <ext-link xlink:href="10.7554/eLife.00001.012">http://dx.doi.org/10.7554/eLife.00001.012</ext-link>',
        ),
        (
            True,
            u'External link rewrite 1 <ext-link ext-link-type="uri" xlink:href="www.r-project.org">www.r-project.org</ext-link>',
            None,
            u'External link rewrite 1 <a href="http://www.r-project.org">www.r-project.org</a>',
        ),
        (
            True,
            u'External link rewrite 2 <ext-link ext-link-type="uri" xlink:href="http://www.r-project.org">www.r-project.org</ext-link>',
            None,
            u'External link rewrite 2 <a href="http://www.r-project.org">www.r-project.org</a>',
        ),
        (
            True,
            u'External link rewrite 3 <ext-link ext-link-type="uri" xlink:href="ftp://example.org">ftp://example.org</ext-link>',
            None,
            u'External link rewrite 3 <a href="ftp://example.org">ftp://example.org</a>',
        ),
        (
            True,
            u'<p>The Panda database (<ext-link ext-link-type="uri" xlink:href="http://circadian.salk.edu/about.html)%20does%20not%20indicate%20restoration%20of%20Cyp2b10">http://circadian.salk.edu/about.html) does not indicate restoration of <italic>Cyp2b10</italic></ext-link> cycling by restricted feeding of clockless mice.</p>',
            None,
            u'<p>The Panda database (<a href="http://circadian.salk.edu/about.html)%20does%20not%20indicate%20restoration%20of%20Cyp2b10">http://circadian.salk.edu/about.html) does not indicate restoration of <i>Cyp2b10</i></a> cycling by restricted feeding of clockless mice.</p>',
        ),
        (True, u"<p>An empty tag <italic/></p>", None, u"<p>An empty tag <i/></p>"),
        (
            True,
            u"<p><email>email@example.org</email></p>",
            None,
            u'<p><a href="mailto:email@example.org">email@example.org</a></p>',
        ),
        (
            True,
            u"<p>A first <email>email@example.org</email> and second <email>another@example.org</email></p>",
            None,
            u'<p>A first <a href="mailto:email@example.org">email@example.org</a> and second <a href="mailto:another@example.org">another@example.org</a></p>',
        ),
        (
            True,
            u'<p><inline-graphic xlink:href="elife-00240-inf1-v1"/></p>',
            None,
            u'<p><img src="elife-00240-inf1-v1.jpg"/></p>',
        ),
        (
            True,
            u'<p><inline-graphic xlink:href="elife-00240-inf1-v1.tiff"/></p>',
            None,
            u'<p><img src="elife-00240-inf1-v1.jpg"/></p>',
        ),
        (
            True,
            u'<p><inline-graphic xlink:href="elife-00240-inf1-v1.tif"/>Some text <inline-graphic xlink:href="elife-00240-inf2-v1.jpg"/>><inline-graphic xlink:href="elife-00240-inf3-v1.gif"></inline-graphic></p>',
            "https://example.org/",
            u'<p><img src="https://example.org/elife-00240-inf1-v1.jpg"/>Some text <img src="https://example.org/elife-00240-inf2-v1.jpg"/>&gt;<img src="https://example.org/elife-00240-inf3-v1.gif"/></p>',
        ),
        (
            True,
            u"<p>Bad inline-graphic for test coverage <inline-graphic/></p>",
            None,
            u"<p>Bad inline-graphic for test coverage <inline-graphic/></p>",
        ),
        (
            True,
            u'<p>Xref tag with multiple rid from 09561 v1 to <xref ref-type="fig" rid="fig3 fig4">Figures 3, 4</xref></p>',
            None,
            u'<p>Xref tag with multiple rid from 09561 v1 to <a href="#fig3">Figures 3, 4</a></p>',
        ),
        (True, u"<break></break>", None, u"<br></br>"),
        (True, u"<monospace>m</monospace>", None, u'<span class="monospace">m</span>'),
        (
            True,
            u"<table><thead><!--  Header row  --><tr><!-- This header row ... --><th></th></tr></thead><tbody><!--  Table body  --><tr><td>Genotype</td></tr></tbody></table>",
            None,
            u"<table><thead><tr><th></th></tr></thead><tbody><tr><td>Genotype</td></tr></tbody></table>",
        ),
        # Replace a particular style pattern with a class name
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="author-callout-style-b8">RS19</td></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td class="author-callout-style-b8">RS19</td></tr></tbody></table>',
        ),
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="author-callout-style-b8"/></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td class="author-callout-style-b8"/></tr></tbody></table>',
        ),
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="author-callout-style-b8" valign="top">RS19</td></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td class="author-callout-style-b8" valign="top">RS19</td></tr></tbody></table>',
        ),
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td valign="top" style="author-callout-style-b8">RS19</td></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td valign="top" class="author-callout-style-b8">RS19</td></tr></tbody></table>',
        ),
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td valign="top"><bold>R143Q</bold></td><td valign="top"/><td style="author-callout-style-b8" valign="top"/><td style="author-callout-style-b8" valign="top"/><td style="author-callout-style-b8" valign="top"/><td style="author-callout-style-b8" valign="top"/><td><bold>-2</bold></td><td valign="top"/></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td valign="top"><b>R143Q</b></td><td valign="top"/><td class="author-callout-style-b8" valign="top"/><td class="author-callout-style-b8" valign="top"/><td class="author-callout-style-b8" valign="top"/><td class="author-callout-style-b8" valign="top"/><td><b>-2</b></td><td valign="top"/></tr></tbody></table>',
        ),
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="author-callout-style-b8">RS19</td><td style="author-callout-style-b8">RS19</td></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td class="author-callout-style-b8">RS19</td><td class="author-callout-style-b8">RS19</td></tr></tbody></table>',
        ),
        # Do not replace general styles with a class name
        (
            True,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="width:51.5pt;border-top:none;border-left:none; border-bottom:dashed; padding:0in 5.4pt 0in 5.4pt">CAP-Gly domain</td></tr></tbody></table>',
            None,
            u'<table><thead><tr><th></th></tr></thead><tbody><tr><td style="width:51.5pt;border-top:none;border-left:none; border-bottom:dashed; padding:0in 5.4pt 0in 5.4pt">CAP-Gly domain</td></tr></tbody></table>',
        ),
        (
            True,
            u'<named-content content-type="author-callout-style-a1">author-callout-style-a1</named-content>',
            None,
            u'<span class="author-callout-style-a1">author-callout-style-a1</span>',
        ),
        (
            True,
            u"<p>Bad named-content for test coverage <named-content/></p>",
            None,
            u"<p>Bad named-content for test coverage <named-content/></p>",
        ),
        # Edge case based on elife article 24634
        (
            True,
            u"<p>Regional analyses identified portions of ... (p<e-200) <italic>MYH7</italic></p>",
            None,
            u"<p>Regional analyses identified portions of ... (p&lt;e-200) <i>MYH7</i></p>",
        ),
        # Edge case with a greater than tag, based on elife article 28862
        (
            True,
            u"<p>We observed significant colocalization (>74%) of <italic>Clensor</italic> with LMP-1-GFP labeled lysosomes ....</p>",
            None,
            u"<p>We observed significant colocalization (&gt;74%) of <i>Clensor</i> with LMP-1-GFP labeled lysosomes ....</p>",
        ),
        # Ampersand test
        (True, u"<p>&</p>", None, u"<p>&amp;</p>"),
        # related-object tag, can be found in a structured abstract
        (
            True,
            (
                '<related-object id="CT1" source-type="clinical-trials-registry" '
                'source-id="ClinicalTrials.gov" source-id-type="registry-name" '
                'document-id="NCT02968459" document-id-type="clinical-trial-number" '
                'xlink:href="https://clinicaltrials.gov/show/NCT02968459">NCT02968459.'
                "</related-object>"
            ),
            None,
            '<a href="https://clinicaltrials.gov/show/NCT02968459">NCT02968459.</a>',
        ),
        # table containing a list
        (
            True,
            u"<table><thead><tr><th></th></tr></thead><tbody><tr><td>Genotype<list></list></td></tr>"
            u"</tbody></table>",
            None,
            u"<table><thead><tr><th></th></tr></thead><tbody><tr><td>Genotype<ul></ul></td></tr>"
            u"</tbody></table>",
        ),
        # table with inline styles and styled-content
        (
            True,
            "<table><thead><tr><th></th><th></th></tr></thead><tbody><tr>"
            '<td><styled-content style="color: #D50000;">400</styled-content></td>'
            '<td style="background-color: #FFB74D;">30503.40</td>'
            "</tr></tbody></table>",
            None,
            "<table><thead><tr><th></th><th></th></tr></thead><tbody><tr>"
            '<td><span style="color: #D50000;">400</span></td>'
            '<td style="background-color: #FFB74D;">30503.40</td>'
            "</tr></tbody></table>",
        ),
    )
    def test_xml_to_html(self, html_flag, xml_string, base_url, expected):
        self.assertEqual(
            utils_html.xml_to_html(html_flag, xml_string, base_url), expected
        )

    def test_replace_related_object_tagsl(self):
        """test when link is not prefaced with an internet scheme prefix"""
        tag_string = (
            '<related-object xlink:href="clinicaltrials.gov/show/NCT02968459">NCT02968459.'
            "</related-object>"
        )
        expected = (
            '<a href="http://clinicaltrials.gov/show/NCT02968459">NCT02968459.</a>'
        )
        self.assertEqual(utils_html.replace_related_object_tags(tag_string), expected)

    @unpack
    @data(
        ("", ""),
        ("<tex-math/>", ""),
    )
    def test_remove_tex_math_tags(self, xml_string, expected):
        "tests for remove_tex_math_tags()"
        self.assertEqual(utils_html.remove_tex_math_tags(xml_string), expected)

    @unpack
    @data(
        ("", ""),
        ("<alternatives/>", "<alternatives/>"),
        (
            '<disp-formula id="equ1"><alternatives/></disp-formula>',
            '<disp-formula id="equ1"></disp-formula>',
        ),
        (
            "<inline-formula><alternatives/></inline-formula>",
            "<inline-formula></inline-formula>",
        ),
    )
    def test_remove_alternatives_tags(self, xml_string, expected):
        "test for remove_alternatives_tags()"
        self.assertEqual(utils_html.remove_alternatives_tags(xml_string), expected)

    @unpack
    @data(
        ("", ""),
        ("<list/>", "<ul/>"),
        ("<list><list-item/></list>", "<ul><li/></ul>"),
        ("<list><list-item>One</list-item></list>", "<ul><li>One</li></ul>"),
        (
            '<list list-type="alpha-lower"><list-item>One</list-item></list>',
            '<ol class="list list--alpha-lower"><li>One</li></ol>',
        ),
        (
            '<list list-type="alpha-upper"><list-item>One</list-item></list>',
            '<ol class="list list--alpha-upper"><li>One</li></ol>',
        ),
        (
            '<list list-type="bullet"><list-item>One</list-item></list>',
            '<ul class="list list--bullet"><li>One</li></ul>',
        ),
        (
            '<list list-type="order"><list-item>One</list-item></list>',
            '<ol class="list list--number"><li>One</li></ol>',
        ),
        (
            '<list list-type="roman-lower"><list-item>One</list-item></list>',
            '<ol class="list list--roman-lower"><li>One</li></ol>',
        ),
        (
            '<list list-type="roman-upper"><list-item>One</list-item></list>',
            '<ol class="list list--roman-upper"><li>One</li></ol>',
        ),
        (
            '<list list-type="simple"><list-item>One</list-item></list>',
            '<ul class="list"><li>One</li></ul>',
        ),
    )
    def test_replace_list_tags(self, xml_string, expected):
        self.assertEqual(utils_html.replace_list_tags(xml_string), expected)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
import os
import time
from collections import OrderedDict
from ddt import ddt, data, unpack
from elifetools import utils
from elifetools import parseJATS as parser
from elifetools.utils_html import allowed_xml_tag_fragments
from tests import soup_body


@ddt
class TestUtils(unittest.TestCase):
    def setUp(self):
        self.mock_tag = type("", (object,), {})
        setattr(self.mock_tag, "name", "foo")

    @unpack
    @data((None, None),
          ([], None),
          (iter([]), None),
          (["bees"], "bees"),
          (["bees", "knees"], "bees"),
          (iter(["bees", "knees"]), "bees")
    )
    def test_first(self, value, expected):
        self.assertEqual(utils.first(value), expected)

    @unpack
    @data(
        (None, None),
        ("this and that", "this-that"),
        ("six of one and half a dozen", "six-one-half-a-dozen"),
    )
    def test_subject_slug_default(self, value, expected):
        "subject_slug with the default stopwords"
        self.assertEqual(utils.subject_slug(value), expected)

    @unpack
    @data(
        (None, None, None),
        ("this and that", [], "this-and-that"),
        ("this and that", ["this", "and", "that"], ""),
    )
    def test_subject_slug_stopwords(self, value, stopwords, expected):
        "subject_slug specifying alternate stopwords"
        self.assertEqual(utils.subject_slug(value, stopwords), expected)

    @unpack
    @data(
        (None, None),
        ("An : ( example ) .", "An: (example)."),
        (["bees .", "( knees )"], ["bees.", "(knees)"]),
    )
    def test_strip_punctuation_space(self, value, expected):
        self.assertEqual(utils.strip_punctuation_space(value), expected)

    @unpack
    @data(
        (None, None, ".", None),
        ("", None, ".", None),
        ("Something", None, ".", "Something"),
        ("Short sentence", "Another", ".", "Short sentence. Another"),
        ("Short sentence ", " Another", ".", "Short sentence. Another"),
        ("Short sentence.", "Another", ".", "Short sentence. Another"),
        ("Short sentence. ", "Another", ".", "Short sentence. Another"),
        # use a comma as the glue just to check it works
        ("Short sentence", "Another", ",", "Short sentence, Another"),
    )
    def test_join_sentences(self, value1, value2, glue, expected):
        self.assertEqual(utils.join_sentences(value1, value2, glue), expected)

    @unpack
    @data(
        (None, None, None),
        ("1", 0xDEADBEEF, 1),
        ("1", "moo", 1),
        ("two", 0xDEADBEEF, "two"),
        ("two", "moo", "moo"),
    )
    def test_coerce_to_int(self, value, default, expected):
        self.assertEqual(utils.coerce_to_int(value, default), expected)

    @unpack
    @data(
        (2015, 6, 22, "UTC", "%Y-%m-%d %Z", "2015-06-22 UTC"),
        ("2015", "06", "22", "UTC", "%Y-%m-%d %Z", "2015-06-22 UTC"),
        ("2015", "06", "31", "UTC", "%Y-%m-%d %Z", None),
        ("2015", "06", "0", "UTC", "%Y-%m-%d %Z", None),
        (None, "06", "22", "UTC", "%Y-%m-%d %Z", None),
    )
    def test_date_struct(self, year, month, day, tz, date_format, date_string_expected):
        expected_date_struct = None
        if date_string_expected:
            expected_date_struct = time.strptime(date_string_expected, date_format)
        self.assertEqual(utils.date_struct(year, month, day, tz), expected_date_struct)

    def test_tag_media_sibling_ordinal_bad_input(self):
        self.assertEqual(utils.tag_media_sibling_ordinal(self.mock_tag), None)

    def test_tag_supplementary_material_sibling_ordinal_bad_input(self):
        self.assertEqual(
            utils.tag_supplementary_material_sibling_ordinal(self.mock_tag), None
        )

    @unpack
    @data(
        ("<p><bold>A</bold> c</p>", None, "<p><bold>A</bold> c</p>"),
        ("<p><bold>A</bold> c</p>", "bold", "<p> c</p>"),
        (
            "<p>A <bold>c</bold> <italic>d</italic> <fig>e</fig></p>",
            ["bold", "fig"],
            "<p>A  <italic>d</italic> </p>",
        ),
    )
    def test_remove_tag_from_tag(self, xml, unwanted_tag_names, expected_xml):
        soup = parser.parse_xml(xml)
        tag = soup.find_all()[0]
        modified_tag = utils.remove_tag_from_tag(tag, unwanted_tag_names)
        self.assertEqual(str(modified_tag), expected_xml)

    def test_convert_testing_doi(self):
        cases = [
            ("10.7554/eLife.00666", "10.7554/eLife.00666"),
            ("10.7554/eLife.507424566981410635", "10.7554/eLife.10635"),
            # component
            ('10.7554/eLife.09560.sa0', '10.7554/eLife.09560.sa0'),
            # versioned
            ('10.7554/eLife.09560.1', '10.7554/eLife.09560.1'),
            ('10.7554/eLife.09560.1.sa0', '10.7554/eLife.09560.1.sa0'),
            # case preserved
            ('10.7554/ELIFE.09560', '10.7554/ELIFE.09560'),
            # unlikely cases
            ('10.7554/eLife.0', '10.7554/eLife.0'),
            ('10.7554/eLife.0.1', '10.7554/eLife.0.1'),
            ('10.7554/eLife.0.1.2.3.4', '10.7554/eLife.0.1.2.3.4'),

            # preserves unparseable strings
            ('', ''),
            ('not_a_doi', 'not_a_doi'),

            # no match cases
            (None, None),
            ([], None),
            ({}, None),
        ]
        for given, expected in cases:
            self.assertEqual(utils.convert_testing_doi(given), expected, "failed case %r" % given)

    @unpack
    @data(
        (None, None),
        ("http://dx.doi.org/10.7554/eLife.00666", "10.7554/eLife.00666"),
        ("https://dx.doi.org/10.7554/eLife.00666", "10.7554/eLife.00666"),
        ("http://doi.org/10.7554/eLife.00666", "10.7554/eLife.00666"),
        ("https://doi.org/10.7554/eLife.00666", "10.7554/eLife.00666"),
    )
    def test_doi_uri_to_doi(self, doi_uri, expected_doi):
        self.assertEqual(utils.doi_uri_to_doi(doi_uri), expected_doi)

    @unpack
    @data(
        (None, None),
        ("10.7554/eLife.00666", "https://doi.org/10.7554/eLife.00666"),
    )
    def test_doi_to_doi_uri(self, doi_uri, expected_doi):
        self.assertEqual(utils.doi_to_doi_uri(doi_uri), expected_doi)

    @unpack
    @data(
        ("<root><p><bold>A</bold> c</p></root>", u"[<p><bold>A</bold> c</p>]"),
        (
            "<root><p><bold>A</bold> c</p><p>DOI:</p></root>",
            u"[<p><bold>A</bold> c</p>]",
        ),
        # example based on unwanted tag in elife 00049
        (
            '<root><p>First paragraph</p><p><bold>A</bold> c</p><p><ext-link ext-link-type="doi" xlink:href="10.7554/eLife.00049.011">http://dx.doi.org/10.7554/eLife.00049.011</ext-link></p></root>',
            u"[<p>First paragraph</p>, <p><bold>A</bold> c</p>]",
        ),
        # example to keep the DOI looking paragraph because there is more content
        (
            '<root><p><bold>A</bold> c</p><p><ext-link ext-link-type="doi" xlink:href="10.7554/eLife.00049.011">http://dx.doi.org/10.7554/eLife.00049.011</ext-link><bold>Content to test.</bold></p></root>',
            u'[<p><bold>A</bold> c</p>, <p><ext-link ext-link-type="doi" href="10.7554/eLife.00049.011">http://dx.doi.org/10.7554/eLife.00049.011</ext-link><bold>Content to test.</bold></p>]',
        ),
    )
    def test_remove_doi_paragraph(self, xml, expected_xml):
        soup = parser.parse_xml(xml)
        modified_tag = utils.remove_doi_paragraph(soup_body(soup))
        self.assertEqual(str(modified_tag), expected_xml)

    @unpack
    @data(
        (None, None),
        ("http://orcid.org/0000-0003-3523-4408", "0000-0003-3523-4408"),
        ("https://orcid.org/0000-0003-3523-4408", "0000-0003-3523-4408"),
    )
    def test_orcid_uri_to_orcid(self, orcid_uri, expected_doi):
        self.assertEqual(utils.orcid_uri_to_orcid(orcid_uri), expected_doi)

    @unpack
    @data(
        (None, None),
        ("A simple example ", u"A simple example"),
        ("Testing one, two, three. Is this thing on?", u"Testing one, two, three"),
        (
            "Predation of <italic>M. sexta</italic> larvae and eggs by <italic>Geocoris</italic> spp. (<bold>A</bold>) Examples of predated <italic>M. sexta</italic> larva (left panel) and egg (right panel).",
            "Predation of <italic>M. sexta</italic> larvae and eggs by <italic>Geocoris</italic> spp.",
        ),
        (
            "Predation of <i>M. sexta</i> larvae and eggs by <i>Geocoris</i> spp. (<b>A</b>) Examples of predated <i>M. sexta</i> larva (left panel) and egg (right panel).",
            "Predation of <i>M. sexta</i> larvae and eggs by <i>Geocoris</i> spp.",
        ),
        (
            "The wild tobacco plant <italic>N. attenuata</italic> relies on both direct and indirect mechanisms to defend it against <italic>M. sexta</italic> caterpillars. Indirect defence involves the release of volatile chemicals that attract <italic>Geocoris</italic> bugs that prey on the caterpillars. This photograph shows a <italic>Geocoris</italic> bug (bottom left) about to attack a caterpillar and two of its larvae.",
            "The wild tobacco plant <italic>N. attenuata</italic> relies on both direct and indirect mechanisms to defend it against <italic>M. sexta</italic> caterpillars",
        ),
        (
            "The unfolded protein response in <italic>S. pombe</italic> and other species. (<bold>A</bold>) The accumulation unfolded proteins in the endoplasmic reticulum (ER) of <italic>S. pombe</italic> leads to activation of IRE1 (presumably by nucleotide binding (green), auto-phosphorylation (red) and the formation of dimers), which is turn leads to the cleavage of mRNAs in the cytosol. The subsequent degradation of the cleaved mRNAs (known as RIDD) and explusion from the cell (via the exosome) reduced the protein-folding load on the ER. However, as described in the text, the mRNA that encodes for the molecular chaperone <italic>Bip1</italic> escapes this fate:",
            "The unfolded protein response in <italic>S. pombe</italic> and other species",
        ),
        (
            'Table summarizing the weights (<italic>w</italic><sub><italic>N</italic></sub>) and performance (expressed as Pearson\'s coefficient <italic>R</italic> and RMSE) of the fourfold cross-validation, repeated 10 times, of the following binding affinity regression model: <disp-formula id="equ4"><mml:math id="m4"><mml:mi>Δ</mml:mi><mml:msub><mml:mtext>G</mml:mtext><mml:mrow><mml:mtext>calc</mml:mtext></mml:mrow></mml:msub><mml:mo>=</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mn>1</mml:mn></mml:msub><mml:msub><mml:mtext>ICs</mml:mtext><mml:mrow><mml:mtext>charged</mml:mtext><mml:mo>/</mml:mo><mml:mtext>charged</mml:mtext></mml:mrow></mml:msub><mml:mo>+</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mn>2</mml:mn></mml:msub><mml:msub><mml:mtext> ICs</mml:mtext><mml:mrow><mml:mtext>charged_apolar</mml:mtext></mml:mrow></mml:msub><mml:mo>−</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mn>3</mml:mn></mml:msub><mml:msub><mml:mtext> ICs</mml:mtext><mml:mrow><mml:mtext>polar</mml:mtext><mml:mo>/</mml:mo><mml:mtext>polar</mml:mtext></mml:mrow></mml:msub><mml:mo>+</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mtext>4</mml:mtext></mml:msub><mml:msub><mml:mtext> ICs</mml:mtext><mml:mrow><mml:mtext>polar</mml:mtext><mml:mo>/</mml:mo><mml:mtext>apolar</mml:mtext></mml:mrow></mml:msub><mml:mo>+</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mn>5</mml:mn></mml:msub><mml:msub><mml:mtext> %NIS</mml:mtext><mml:mrow><mml:mtext>apolar</mml:mtext></mml:mrow></mml:msub><mml:mo>+</mml:mo><mml:msub><mml:mtext>w</mml:mtext><mml:mn>6</mml:mn></mml:msub><mml:msub><mml:mtext> %NIS</mml:mtext><mml:mrow><mml:mtext>charged</mml:mtext></mml:mrow></mml:msub><mml:mo>+</mml:mo><mml:mtext>Q</mml:mtext><mml:mo>.</mml:mo></mml:math></disp-formula>Each coefficient has been reported as average on the four models trained on the respective folds.',
            "Table summarizing the weights (<italic>w</italic><sub><italic>N</italic></sub>) and performance (expressed as Pearson's coefficient <italic>R</italic> and RMSE) of the fourfold cross-validation, repeated 10 times, of the following binding affinity regression model",
        ),
    )
    def test_text_to_title(self, value, expected_title):
        self.assertEqual(utils.text_to_title(value), expected_title)

    @unpack
    @data(
        (None, None),
        (u"", u""),
        (u"\nText\n ", u"Text"),
        (
            u"\nAn example <ext-link>link</i>\n<ext-link>link 2</ext-link>\n",
            u"An example <ext-link>link</i> <ext-link>link 2</ext-link>",
        ),
    )
    def test_clean_whitespace(self, value, expected):
        self.assertEqual(expected, utils.clean_whitespace(value))

    @unpack
    @data(
        (None, None),
        (u"Figure 8", u"Figure 8"),
        (u"Reviewers’ figure 1", u"Reviewers’ figure 1"),
        (u"Figure 7—figure supplement 1:", u"Figure 7—figure supplement 1"),
        (
            u"Appendix 1—figure 3—figure supplement 1.",
            u"Appendix 1—figure 3—figure supplement 1",
        ),
    )
    def test_rstrip_punctuation(self, value, expected):
        self.assertEqual(expected, utils.rstrip_punctuation(value))

    @unpack
    @data(
        (None, None),
        (
            "<p><italic><bold><sup><sub><sc><underline><bold>superstyling</bold></underline></sc></sub></sup></bold></italic></p>",
            "<p><italic><bold><sup><sub><sc><underline><bold>superstyling</bold></underline></sc></sub></sup></bold></italic></p>",
        ),
        ("<", "&lt;"),
        (
            "< T << G << C >> A <<italic>m</italic>",
            "&lt; T &lt;&lt; G &lt;&lt; C &gt;&gt; A &lt;<italic>m</italic>",
        ),
        (
            "<p>**p<0.01; ***p<0.001. SI, aged mice >5 months old.</p>",
            "<p>**p&lt;0.01; ***p&lt;0.001. SI, aged mice &gt;5 months old.</p>",
        ),
    )
    def test_escape_unmatched_angle_brackets(self, value, expected):
        """
        Test some additional examples of unmatched angle brackets specifically
        """
        self.assertEqual(
            utils.escape_unmatched_angle_brackets(value, allowed_xml_tag_fragments()),
            expected,
        )

    @unpack
    @data(
        (None, None),
        ("", ""),
        ("a", "a"),
        ("another & another & another", "another &amp; another &amp; another"),
        ("a&a", "a&amp;a"),
        ("a&amp;&a", "a&amp;&amp;a"),
        ("a&#x0117;a", "a&#x0117;a"),
        (
            "fake link http://example.org/?a=b&amp",
            "fake link http://example.org/?a=b&amp;amp",
        ),
        ("CUT&RUN is performed", "CUT&amp;RUN is performed"),
        ("&#8220;&what?;&#8221; &#169; Anon", "&#8220;&amp;what?;&#8221; &#169; Anon"),
    )
    def test_escape_ampersand(self, value, expected):
        self.assertEqual(utils.escape_ampersand(value), expected)

    @unpack
    @data(
        (None, None, ""),
        ("p", '<p class="p1">Paragraph.</p>', "Paragraph."),
        ("bold", "<p>\n<bold>One<bold> <bold>two</bold>\n</p>", "<p>One two</p>"),
        (
            "italic",
            "<p><italic><italic>Very italic.</italic></italic></p>",
            "<p>Very italic.</p>",
        ),
    )
    def test_remove_tag(self, tag_name, string, expected):
        self.assertEqual(utils.remove_tag(tag_name, string), expected)

    @unpack
    @data(
        (None, None, ""),
        ("p", '<p class="p1">Paragraph.</p>', ""),
        ("italic", "<p><italic><italic>Very italic.</italic></italic></p>", "<p></p>"),
    )
    def test_remove_tag_and_text(self, tag_name, string, expected):
        self.assertEqual(utils.remove_tag_and_text(tag_name, string), expected)

    @unpack
    @data(
        (None, None),
        ("", None),
        ("<root></root>", None),
        (
            "<root><p>Content <!-- comments --></p></root>",
            "<p>Content <!-- comments --></p>",
        ),
    )
    def test_node_contents_str(self, xml, expected):
        if not xml:
            # to test with blank values
            tag = xml
        else:
            # parse the XML into tags to test with
            soup = parser.parse_xml(xml)
            tag = soup_body(soup)
        self.assertEqual(utils.node_contents_str(tag), expected)

    @unpack
    @data(
        (None, "none"),
        ("alpha-lower", "alpha-lower"),
        ("alpha-upper", "alpha-upper"),
        ("bullet", "bullet"),
        ("order", "number"),
        ("roman-lower", "roman-lower"),
        ("roman-upper", "roman-upper"),
        ("simple", "none"),
    )
    def test_list_type_prefix(self, list_type, expected):
        self.assertEqual(utils.list_type_prefix(list_type), expected)

class TestUtilsPlain(unittest.TestCase):
    def test_peek_empty(self):
        "peeking into an empty sequence yields a None"
        self.assertEqual(utils.peek(iter([])), None)

    def test_peek_single(self):
        "peeking into a sequence with a single value yields a pair of: the single value and a new sequence of the same value"
        first, first_and_rest = utils.peek(iter([1]))
        self.assertEqual(first, 1)
        self.assertEqual(list(first_and_rest), [1])

    def test_peek_multi(self):
        "peeking into a sequence with multiple values yields a pair of: the first value and a new sequence of the old values."
        first, first_and_rest = utils.peek(iter([1, 2, 3, 4]))
        self.assertEqual(first, 1)
        self.assertEqual(list(first_and_rest), [1, 2, 3, 4])

class TestUtilsNameFormatting(unittest.TestCase):
    def setUp(self):
        self.surname = "Doe"
        self.given_names = "Jane"
        self.suffix = "Jnr"

    def test_references_author_person(self):
        author = {
            "surname": self.surname,
            "given-names": self.given_names,
            "suffix": self.suffix,
        }
        expected = OrderedDict(
            [
                ("type", "person"),
                (
                    "name",
                    OrderedDict(
                        [("preferred", "Doe Jane Jnr"), ("index", "Doe, Jane, Jnr")]
                    ),
                ),
            ]
        )
        self.assertEqual(utils.references_author_person(author), expected)

    def test_reference_author_preferred_name(self):
        expected = "Doe Jane Jnr"
        self.assertEqual(
            utils.reference_author_preferred_name(
                self.surname, self.given_names, self.suffix
            ),
            expected,
        )

    def test_author_preferred_name(self):
        expected = "Jane Doe Jnr"
        self.assertEqual(
            utils.author_preferred_name(self.surname, self.given_names, self.suffix),
            expected,
        )

    def test_author_index_name(self):
        expected = "Doe, Jane, Jnr"
        self.assertEqual(
            utils.author_index_name(self.surname, self.given_names, self.suffix),
            expected,
        )


class TestNamedEntityToUnicode(unittest.TestCase):
    "tests for named_entity_to_unicode()"

    def test_named_entity_to_unicode(self):
        "test converting a named HTML entity to unicode"
        string = "&alpha;-helix &#x03B1;-helix"
        expected = "\u03b1-helix &#x03B1;-helix"
        # invoke
        result = utils.named_entity_to_unicode(string)
        # assert
        self.assertEqual(result, expected)

    def test_none(self):
        "test None input for coverage"
        self.assertEqual(utils.named_entity_to_unicode(None), None)


class TestEntityToUnicode(unittest.TestCase):
    "tests for entity_to_unicode()"

    def test_entity_to_unicode(self):
        passes = []
        passes.append(
            (
                "N-terminal &#x03B1;-helix into the heterodimer interface",
                "N-terminal \u03b1-helix into the heterodimer interface",
            )
        )

        passes.append(
            (
                "N-terminal &alpha;-helix into the heterodimer interface",
                "N-terminal \u03b1-helix into the heterodimer interface",
            )
        )

        passes.append(
            (
                (
                    "&#x00A0; &#x00C5; &#x00D7; &#x00EF; &#x0394; &#x03B1; &#x03B2; &#x03B3; &#x03BA; "
                    + "&#x03BB; &#x2212; &#x223C; &alpha; &amp; &beta; &epsilon; &iuml; &ldquo; &ordm; "
                    + "&rdquo;"
                ),
                (
                    "\xa0 \xc5 \xd7 \xef \u0394 \u03b1 \u03b2 \u03b3 \u03ba \u03bb \u2212 \u223c "
                    + '\u03b1 &amp; \u03b2 \u03b5 \xcf " \xba "'
                ),
            )
        )

        for string_input, string_output in passes:
            self.assertEqual(utils.entity_to_unicode(string_input), string_output)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
import os
from ddt import ddt, data, unpack
import copy
import elifetools.parseJATS as parser
import elifetools.json_rewrite as json_rewrite


@ddt
class TestJsonRewrite(unittest.TestCase):
    def setUp(self):
        pass

    @unpack
    @data(
        ("", None, None, None),
        (
            "",
            '<root><journal-meta><journal-id journal-id-type="publisher-id">eLife</journal-id></journal-meta></root>',
            None,
            None,
        ),
        (
            "not_a_rewrite_function",
            '<root><journal-meta><journal-id journal-id-type="publisher-id">eLife</journal-id></journal-meta><article-meta><article-id pub-id-type="doi">10.7554/eLife.00051</article-id></article-meta></root>',
            None,
            None,
        ),
    )
    def test_rewrite_json(self, rewrite_type, xml_content, json_content, expected):
        soup = None
        if xml_content:
            soup = parser.parse_xml(xml_content)
        self.assertEqual(
            json_rewrite.rewrite_json(rewrite_type, soup, json_content), expected
        )

    @unpack
    @data(
        (None, None, None),
        ("elife", "func", "rewrite_elife_func"),
        ("eLife", "func", "rewrite_elife_func"),
        ("other_journal", "func", "rewrite_other_journal_func"),
    )
    def test_rewrite_function_name(self, journal_id, rewrite_type, expected):
        self.assertEqual(
            json_rewrite.rewrite_function_name(journal_id, rewrite_type), expected
        )

    @unpack
    @data(
        ({"used": [{"id": "dataro17"}]}, "10.7554/eLife.00348"),
        ({"used": [{"id": "dataro3"}]}, "10.7554/eLife.01311"),
        ({"used": [{"id": "dataro6"}]}, "10.7554/eLife.01311"),
        ({"used": [{"id": "dataro7"}]}, "10.7554/eLife.01311"),
        ({"used": [{"id": "dataro8"}]}, "10.7554/eLife.01311"),
        ({"used": [{"id": "dataro1"}]}, "10.7554/eLife.01440"),
        ({"used": [{"id": "dataro1", "date": "2000, 2005"}]}, "10.7554/eLife.01535"),
        ({"used": [{"id": "dataro11"}]}, "10.7554/eLife.02304"),
        ({"used": [{"id": "dataro2"}]}, "10.7554/eLife.03574"),
        ({"used": [{"id": "dataro4"}]}, "10.7554/eLife.03676"),
        ({"used": [{"id": "dataro2"}]}, "10.7554/eLife.03971"),
        (
            {"generated": [{"id": "dataro1", "date": "2014-2015"}]},
            "10.7554/eLife.04660",
        ),
        ({"used": [{"id": "dataro2", "date": "NA"}]}, "10.7554/eLife.06421"),
        ({"used": [{"id": "data-ro1"}]}, "10.7554/eLife.08445"),
        (
            {"used": [{"id": "dataro2", "date": "2008, updated 2014"}]},
            "10.7554/eLife.08916",
        ),
        (
            {"used": [{"id": "dataro3", "date": "2013, updated 2014"}]},
            "10.7554/eLife.08916",
        ),
        ({"generated": [{"id": "dataro2"}]}, "10.7554/eLife.08955"),
        ({"used": [{"id": "dataro1"}]}, "10.7554/eLife.09207"),
        ({"generated": [{"id": "data-ro4"}]}, "10.7554/eLife.10607"),
        ({"used": [{"id": "data-ro1"}]}, "10.7554/eLife.10670"),
        ({"generated": [{"id": "dataro7"}]}, "10.7554/eLife.10856"),
        ({"generated": [{"id": "dataro8"}]}, "10.7554/eLife.10856"),
        ({"generated": [{"id": "dataro9"}]}, "10.7554/eLife.10856"),
        ({"generated": [{"id": "dataro1"}]}, "10.7554/eLife.10877"),
        ({"generated": [{"id": "dataro1"}]}, "10.7554/eLife.10921"),
        ({"used": [{"id": "dataro2"}]}, "10.7554/eLife.10921"),
        ({"used": [{"id": "dataro14"}]}, "10.7554/eLife.11117"),
        ({"used": [{"id": "dataro1"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro2"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro3"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro4"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro5"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro6"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro7"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro8"}]}, "10.7554/eLife.12204"),
        ({"used": [{"id": "dataro1"}]}, "10.7554/eLife.12876"),
        ({"generated": [{"id": "dataro1"}]}, "10.7554/eLife.13195"),
        ({"generated": [{"id": "data-ro1"}]}, "10.7554/eLife.14158"),
        ({"generated": [{"id": "data-ro2"}]}, "10.7554/eLife.14158"),
        ({"used": [{"id": "dataro3"}]}, "10.7554/eLife.14158"),
        ({"generated": [{"id": "dataro2"}]}, "10.7554/eLife.14243"),
        (
            {"generated": [{"id": "dataro1", "date": "current manuscript"}]},
            "10.7554/eLife.16078",
        ),
        ({"used": [{"id": "data-ro4"}]}, "10.7554/eLife.17082"),
        ({"used": [{"id": "data-ro5"}]}, "10.7554/eLife.17082"),
        ({"used": [{"id": "data-ro6"}]}, "10.7554/eLife.17082"),
        (
            {"generated": [{"id": "dataro1", "date": "Release date: "}]},
            "10.7554/eLife.17473",
        ),
    )
    def test_rewrite_elife_datasets_json(self, json_content, doi):
        """simple tests for coverage assert the result is different"""
        original_json_content = copy.deepcopy(json_content)
        self.assertNotEqual(
            json_rewrite.rewrite_elife_datasets_json(json_content, doi),
            original_json_content,
        )

    @unpack
    @data(
        ([{}], "10.7554/eLife.00001", [{}]),
        (
            [{}],
            "10.7554/eLife.00230",
            [
                {
                    "competingInterests": "The authors have declared that no competing interests exist"
                }
            ],
        ),
        (
            [
                {"name": {"index": "Chen, Zhijian J"}},
                {"name": {"index": "Li, Xiao-Dong"}},
            ],
            "10.7554/eLife.00102",
            [
                {
                    "name": {"index": "Chen, Zhijian J"},
                    "competingInterests": "ZJC: Reviewing Editor, <i>eLife</i>",
                },
                {
                    "name": {"index": "Li, Xiao-Dong"},
                    "competingInterests": "No competing interests declared.",
                },
            ],
        ),
        (
            [{"name": {"index": "Patterson, Mark"}}],
            "10.7554/eLife.00270",
            [
                {
                    "name": {"index": "Patterson, Mark"},
                    "competingInterests": "MP: Managing Executive Editor, <i>eLife</i>",
                }
            ],
        ),
        (
            [{}],
            "10.7554/eLife.507424566981410635",
            [
                {
                    "competingInterests": "The authors declare that no competing interests exist."
                }
            ],
        ),
    )
    def test_rewrite_elife_authors_json(self, json_content, doi, expected):
        original_json_content = copy.deepcopy(json_content)
        self.assertEqual(
            json_rewrite.rewrite_elife_authors_json(json_content, doi), expected
        )

    @unpack
    @data(
        ([{}], "10.7554/eLife.23804", [{"role": "Reviewing Editor"}]),
        (
            [{"role": "Reviewing editor"}],
            "10.7554/eLife.00001",
            [{"role": "Reviewing Editor"}],
        ),
        (
            [{"role": "Specialised Role"}],
            "10.7554/eLife.00001",
            [{"role": "Specialised Role"}],
        ),
    )
    def test_rewrite_elife_editors_json(self, json_content, doi, expected):
        """simple tests for coverage assert the result is different"""
        original_json_content = copy.deepcopy(json_content)
        self.assertEqual(
            json_rewrite.rewrite_elife_editors_json(json_content, doi), expected
        )


if __name__ == "__main__":
    unittest.main()



import os
import unittest
import bs4
from ddt import ddt, data, unpack
from elifetools import parseJATS as parser
from elifetools import rawJATS as raw_parser
from tests.file_utils import sample_xml, read_fixture


CACHED_SOUP_MAP = {}


def get_cached_soup(filename):
    if filename not in CACHED_SOUP_MAP:
        CACHED_SOUP_MAP[filename] = parser.parse_document(sample_xml(filename))
    return CACHED_SOUP_MAP[filename]


@ddt
class TestJatsParser(unittest.TestCase):
    def setUp(self):
        self.kitchen_sink_xml = sample_xml("elife-kitchen-sink.xml")
        self.xml = sample_xml("elife00013.xml")

    def tearDown(self):
        pass

    def test_quickly(self):
        struct = [
            (parser.doi, u"10.7554/eLife.00013"),
            (parser.journal_id, u"eLife"),
            (parser.journal_title, u"eLife"),
            (parser.journal_issn, u"2050-084X"),
            (parser.publisher, u"eLife Sciences Publications, Ltd"),
        ]
        for func, expected in struct:
            soup = parser.parse_document(self.kitchen_sink_xml)
            got = func(soup)
            try:

                self.assertEqual(got, expected)
            except AssertionError:
                print("failed on", func, "expected", expected, "got", got)
                raise
        soup = parser.parse_document(self.kitchen_sink_xml)
        self.assertEqual(
            parser.journal_issn(soup, pub_format="electronic"), u"2050-084X"
        )

    def test_quickly2(self):
        soup = parser.parse_document(self.xml)
        self.assertEqual(raw_parser.article_type(soup), "research-article")

    @unpack
    @data(
        ("elife-kitchen-sink.xml", bs4.element.Tag),
        ("elife_poa_e06828.xml", bs4.element.Tag),
        ("elife07586.xml", bs4.element.Tag),
    )
    def test_back(self, filename, expected_type):
        soup = get_cached_soup(filename)
        self.assertEqual(type(raw_parser.back(soup)), expected_type)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 2),
        ("elife_poa_e06828.xml", 1),
        ("elife07586.xml", 0),
    )
    def test_abstract(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.abstract(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 4),
        ("elife_poa_e06828.xml", 2),
        ("elife07586.xml", 2),
    )
    def test_article_id(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.article_id(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 1),
        ("elife_poa_e06828.xml", 1),
        ("elife07586.xml", 1),
    )
    def test_meta_article_id(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.meta_article_id(soup, "doi")), expected_len)

    @unpack
    @data(
        (
            "elife-kitchen-sink.xml",
            '<article-id pub-id-type="doi">10.7554/eLife.00013</article-id>',
        ),
        (
            "elife_poa_e06828.xml",
            '<article-id pub-id-type="doi">10.7554/eLife.06828</article-id>',
        ),
        (
            "elife07586.xml",
            '<article-id pub-id-type="doi">10.7554/eLife.07586</article-id>',
        ),
    )
    def test_doi(self, filename, expected):
        soup = get_cached_soup(filename)
        self.assertEqual(str(raw_parser.doi(soup)), expected)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", "None"),
        ("elife_poa_e06828.xml", "None"),
        ("elife07586.xml", "None"),
    )
    def test_version_doi(self, filename, expected):
        soup = get_cached_soup(filename)
        self.assertEqual(str(raw_parser.version_doi(soup)), expected)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 3),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 1),
    )
    def test_body(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.body(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", bs4.element.Tag),
        ("elife_poa_e06828.xml", type(None)),
        ("elife07586.xml", bs4.element.Tag),
    )
    def test_article_body(self, filename, expected_type):
        soup = get_cached_soup(filename)
        self.assertEqual(type(raw_parser.article_body(soup)), expected_type)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 2),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_sub_article(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.sub_article(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", bs4.element.Tag),
        ("elife_poa_e06828.xml", type(None)),
        ("elife07586.xml", type(None)),
        ("elife-00666.xml", bs4.element.Tag),
    )
    def test_decision_letter(self, filename, expected_type):
        soup = get_cached_soup(filename)
        self.assertEqual(type(raw_parser.decision_letter(soup)), expected_type)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", bs4.element.Tag),
        ("elife_poa_e06828.xml", type(None)),
        ("elife07586.xml", type(None)),
    )
    def test_author_response(self, filename, expected_type):
        soup = get_cached_soup(filename)
        self.assertEqual(type(raw_parser.author_response(soup)), expected_type)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 25),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_section(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.section(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 177),
        ("elife_poa_e06828.xml", 3),
        ("elife07586.xml", 5),
    )
    def test_paragraph(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.paragraph(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 4),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_table(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.table(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 2),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_table_wrap_foot(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.table_wrap_foot(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 6),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_disp_formula(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.disp_formula(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 20),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 0),
    )
    def test_math(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.math(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", 4),
        ("elife_poa_e06828.xml", 0),
        ("elife07586.xml", 1),
    )
    def test_boxed_text(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.boxed_text(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 25), ("elife_poa_e06828.xml", 0))
    def test_fig(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.fig(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 3), ("elife_poa_e06828.xml", 0))
    def test_fig_group(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.fig_group(soup)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", None, 2),
        ("elife-kitchen-sink.xml", "executive-summary", 1),
        ("elife_poa_e06828.xml", None, 1),
        ("elife_poa_e06828.xml", "executive-summary", 0),
    )
    def test_abstract_with_abstract_type(self, filename, abstract_type, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.abstract(soup, abstract_type)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 0), ("elife-02833-v2.xml", 1))
    def test_list(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.get_list(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 0), ("elife-02833-v2.xml", 4))
    def test_list_item(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.list_item(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", "doi", 78), ("elife-kitchen-sink.xml", None, 79))
    def test_pub_id_doi(self, filename, pub_id_type, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.pub_id(soup, pub_id_type)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", "uri", 7), ("elife-kitchen-sink.xml", None, 47))
    def test_pub_id_uri(self, filename, ext_link_type, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.ext_link(soup, ext_link_type)), expected_len)

    @unpack
    @data(
        ("elife-kitchen-sink.xml", "ethics-information", 1),
        ("elife-kitchen-sink.xml", None, 3),
    )
    def test_fn_group(self, filename, content_type, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.fn_group(soup, content_type)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 1))
    def test_app_group(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.app_group(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 2))
    def test_app(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.app(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 1))
    def test_funding_group(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.funding_group(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 7))
    def test_award_group(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.award_group(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 7))
    def test_principal_award_recipient(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.principal_award_recipient(soup)), expected_len)

    @unpack
    @data(("elife-kitchen-sink.xml", 1))
    def test_mixed_citations(self, filename, expected_len):
        soup = get_cached_soup(filename)
        self.assertEqual(len(raw_parser.mixed_citations(soup)), expected_len)

    @unpack
    @data(
        (read_fixture("test_pub_history", "content_01.xml"), 1),
    )
    def test_pub_history(self, xml_content, expected_len):
        soup = parser.parse_xml(xml_content)
        self.assertEqual(len(raw_parser.pub_history(soup)), expected_len)

    @unpack
    @data(
        (read_fixture("test_pub_history", "content_01.xml"), 1),
    )
    def test_raw_event(self, xml_content, expected_len):
        soup = parser.parse_xml(xml_content)
        self.assertEqual(len(raw_parser.event(soup)), expected_len)

    @unpack
    @data(
        (read_fixture("test_pub_history", "content_01.xml"), 1),
    )
    def test_event_desc(self, xml_content, expected_len):
        soup = parser.parse_xml(xml_content)
        self.assertEqual(len(raw_parser.event_desc(soup)), expected_len)


# some XML content for testing sub-article parsing functions
SUB_ARTICLE_XML_CONTENT = read_fixture("", "sub_articles.xml")


class TestFilterSubArticles(unittest.TestCase):
    def setUp(self):
        # sample data with a variety of XML data with an abbreviated structure for testing
        soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)
        self.nodes = raw_parser.sub_article(soup)

    def test_filter_none(self):
        expected_len = 0
        self.assertEqual(
            len(raw_parser.filter_sub_articles_by_title(None)), expected_len
        )

    def test_filter_blank(self):
        expected_len = 0
        self.assertEqual(len(raw_parser.filter_sub_articles_by_title([])), expected_len)

    def test_filter_no_article_title(self):
        "test when a node has no article title"
        xml_content = "<root><sub-article/></root>"
        expected_len = 1
        soup = parser.parse_xml(xml_content)
        nodes = raw_parser.sub_article(soup)
        self.assertEqual(
            len(raw_parser.filter_sub_articles_by_title(nodes)), expected_len
        )

    def test_filter_default(self):
        expected_len = len(self.nodes)
        self.assertEqual(
            len(raw_parser.filter_sub_articles_by_title(self.nodes)), expected_len
        )

    def test_filter_include_title(self):
        include_title = "recommendations"
        expected_len = 1
        self.assertEqual(
            len(
                raw_parser.filter_sub_articles_by_title(
                    self.nodes, include_title=include_title
                )
            ),
            expected_len,
        )

    def test_filter_exclude_title(self):
        exclude_title = "recommendations"
        expected_len = len(self.nodes) - 1
        self.assertEqual(
            len(
                raw_parser.filter_sub_articles_by_title(
                    self.nodes, exclude_title=exclude_title
                )
            ),
            expected_len,
        )

    def test_filter_include_and_exclude(self):
        include_title = "evaluation"
        exclude_title = "assessment"
        expected_len = 1
        self.assertEqual(
            len(
                raw_parser.filter_sub_articles_by_title(
                    self.nodes, include_title=include_title, exclude_title=exclude_title
                )
            ),
            expected_len,
        )


class TestEditorEvaluation(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_editor_evaluation(self):
        node = raw_parser.editor_evaluation(self.soup)
        self.assertEqual(node.name, "sub-article")
        self.assertEqual(
            str(raw_parser.article_title(node)),
            "<article-title>Editor's evaluation</article-title>",
        )


class TestElifeAssessment(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_elife_assessment(self):
        node = raw_parser.elife_assessment(self.soup)
        self.assertEqual(node.name, "sub-article")
        self.assertEqual(
            str(raw_parser.article_title(node)),
            "<article-title>eLife assessment</article-title>",
        )


class TestDecisionLetter(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_decision_letter(self):
        node = raw_parser.decision_letter(self.soup)
        self.assertEqual(node.name, "sub-article")
        self.assertEqual(
            str(raw_parser.article_title(node)),
            "<article-title>Decision letter</article-title>",
        )


class TestAuthorResponse(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_author_response(self):
        node = raw_parser.author_response(self.soup)
        self.assertEqual(node.name, "sub-article")
        self.assertEqual(
            str(raw_parser.article_title(node)),
            "<article-title>Author response</article-title>",
        )


class TestRecommendationsForAuthors(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_recommendations_for_authors(self):
        node = raw_parser.recommendations_for_authors(self.soup)
        self.assertEqual(node.name, "sub-article")
        self.assertEqual(
            str(raw_parser.article_title(node)),
            "<article-title>Recommendations for authors</article-title>",
        )


class TestPublicReviews(unittest.TestCase):
    def setUp(self):
        self.soup = parser.parse_xml(SUB_ARTICLE_XML_CONTENT)

    def test_public_reviews(self):
        expected_len = 3
        nodes = raw_parser.public_reviews(self.soup)
        self.assertEqual(len(nodes), expected_len)
        self.assertEqual(
            str(raw_parser.article_title(nodes[0])),
            "<article-title>Reviewer #1 (public review)</article-title>",
        )


if __name__ == "__main__":
    unittest.main()
